/*
 * Moonshine - a Lua virtual machine.
 *
 * Email: moonshine@gamesys.co.uk
 * http://moonshinejs.org
 *
 * Copyright (c) 2013-2015 Gamesys Limited. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @fileOverview The Moonshine Distillery. Converts Lua byte code to JSON.
 * @author <a href="mailto:paul.cuthbertson@gamesys.co.uk">Paul Cuthbertson</a>
 */

"use strict";var shine=shine||{};!function(){function t(){this._data=null,this._pointer=null,this._tree=null}var e=0,r=1,i=3,n=4;t.prototype.parse=function(t,e,r){void 0===r&&(r=e,e={}),this._runConfig=e||{};var i,n,s=this;if(t.substr(0,4)==String.fromCharCode(27,76,117,97)){if(i=t.charCodeAt(4).toString(16),"51"!=i)throw new SyntaxError("The specified file was compiled with Lua v"+i[0]+"."+i[1]+"; Moonshine can only parse bytecode created using the Lua v5.1 compiler.");return this._parseData(t),r&&r(this._tree),this._tree}n=require("fs"),n.readFile(t,"binary",function(t,e){if(t)throw t;s._parseData(""+e),r&&r(s._tree)})},t.prototype._parseData=function(t){this._data=t,this._pointer=0,this._readGlobalHeader(),this._tree=this._readChunk(),delete this._runConfig},t.prototype.getTree=function(){return this._tree},t.prototype._readGlobalHeader=function(){this._config={signature:this._readByte(4),version:this._readByte().toString(16).split("",2).join("."),formatVersion:this._readByte(),endianness:this._readByte(),sizes:{"int":this._readByte(),size_t:this._readByte(),instruction:this._readByte(),number:this._readByte()},integral:this._readByte()}},t.prototype._readByte=function(t){return void 0===t?255&this._data.charCodeAt(this._pointer++):(t=t||1,this._data.substr((this._pointer+=t)-t,t))},t.prototype._readString=function(){var t,e,r,i=this._readByte(this._config.sizes.size_t),n=0;if(this._config.endianness)for(e=this._config.sizes.size_t-1;e>=0;e--)n=256*n+i.charCodeAt(e);else for(e=0,r=this._config.sizes.size_t;r>e;e++)n=256*n+i.charCodeAt(e);return n?(t=this._readByte(n),0==t.charCodeAt(n-1)&&(t=t.substr(0,n-1)),t):""},t.prototype._readInteger=function(){for(var t,e,r,i=this._readByte(this._config.sizes["int"]),n="",e=0,r=i.length;r>e;e++)t=("0"+i.charCodeAt(e).toString(16)).substr(-2),n=this._config.endianness?t+n:n+t;return parseInt(n,16)},t.prototype._readNumber=function(){for(var e=this._readByte(this._config.sizes.number),r="",i=0,n=e.length;n>i;i++)r=("0000000"+e.charCodeAt(i).toString(2)).substr(-8)+r;var s=parseInt(r.substr(-64,1),2),a=parseInt(r.substr(-63,11),2),o=t.binFractionToDec(r.substr(-52,52),2);return 0===a?0:2047===a?1/0:Math.pow(-1,s)*(1+o)*Math.pow(2,a-1023)},t.binFractionToDec=function(t){for(var e=0,r=0,i=t.length;i>r;r++)"1"===t.substr(r,1)&&(e+=1/Math.pow(2,r+1));return e},t.prototype._readInstruction=function(){return this._readByte(this._config.sizes.instruction)},t.prototype._readConstant=function(){var t=this._readByte();switch(t){case e:return;case r:return!!this._readByte();case i:return this._readNumber();case n:return this._readString();default:throw new Error("Unknown constant type: "+t)}},t.prototype._readInstructionList=function(){var t,e=this._readInteger(),r=[];for(t=0;e>t;t++)r.push(this._readInstruction());return r},t.prototype._readConstantList=function(){var t,e=this._readInteger(),r=[];for(t=0;e>t;t++)r.push(this._readConstant());return r},t.prototype._readFunctionList=function(){var t,e=this._readInteger(),r=[];for(t=0;e>t;t++)r.push(this._readChunk());return r},t.prototype._readStringList=function(){var t,e=this._readInteger(),r=[];for(t=0;e>t;t++)r.push(this._readString());return r},t.prototype._readIntegerList=function(){var t,e=this._readInteger(),r=[];for(t=0;e>t;t++)r.push(this._readInteger());return r},t.prototype._readLocalsList=function(){var t,e=this._readInteger(),r=[];for(t=0;e>t;t++)r.push({varname:this._readString(),startpc:this._readInteger(),endpc:this._readInteger()});return r},t.prototype._readChunk=function(){var t={sourceName:this._readString(),lineDefined:this._readInteger(),lastLineDefined:this._readInteger(),upvalueCount:this._readByte(),paramCount:this._readByte(),is_vararg:this._readByte(),maxStackSize:this._readByte(),instructions:this._parseInstructions(this._readInstructionList()),constants:this._readConstantList(),functions:this._readFunctionList(),linePositions:this._readIntegerList(),locals:this._readLocalsList(),upvalues:this._readStringList()};return this._runConfig.stripDebugging&&(delete t.linePositions,delete t.locals,delete t.upvalues),t},t.prototype._parseInstructions=function(t){for(var e=[],r=0,i=t.length;i>r;r++)e.push.apply(e,this._parseInstruction(t[r]));return e},t.prototype._parseInstruction=function(t){var e,r,i=0,n=[0,0,0,0];if(this._config.endianness)for(e=t.length;e>=0;e--)i=(i<<8)+t.charCodeAt(e);else for(e=0,r=t.length;r>e;e++)i=(i<<8)+t.charCodeAt(e);switch(n[0]=63&i,n[1]=i>>6&255,n[0]){case 1:case 5:case 7:case 36:n[2]=i>>14&16383;break;case 22:case 31:case 32:n[2]=(i>>>14)-131071;break;default:n[2]=i>>23&511,n[3]=i>>14&511}return this._runConfig.useInstructionObjects?[{op:n[0],A:n[1],B:n[2],C:n[3]}]:n},shine.distillery={Parser:t}}(),"undefined"!=typeof module&&(module.exports=shine.distillery);
